# coding: utf-8
u"""Данный модуль является IoC-контейнером.

Используется для связывания объектов и классов проекта с общей кодовой базой.
При необходимости использования в общей кодовой базе объектов проекта,
нужно добавить переменные со значением _empty в данный модуль.
"""

# Пустой класс (Sentinel), чтобы не присваивать None, т.к. None может быть
# хранимым значением.
_empty = type('', tuple(), dict())

observer = _empty
u"""Реестр слушателей системы."""

auth_controller = _empty
u"""Контроллер, обеспечивающий работу с экшенами авторизации."""

roles_controller = _empty
u"""Контроллер, обеспечивающий работу с экшенами реестра ролей RBAC."""

get_current_user = _empty
u"""Функция для получения текущего пользователя, которому назначаются роли.

Должна реализовываться в проекте в связи с тем, что роли могут закрепляться за
любыми моделями проекта (пользователями, физ. лицами, сотрудниками, учениками,
родителями и т.д.).
"""


def register(name, value):
    u"""Регистрирует в контейнере значение *value* под именем *name*.

    *name* должен быть объявлен в модуле и равен *_empty*.
    """
    assert name in globals(), name
    assert globals()[name] is _empty, (
        u'"{}" always registered'.format(name)
    )

    globals()[name] = value

    return value


def get(name):
    u"""Возвращает содержимое с именем *name*.

    *name* должен быть предварительно зарегистрирован через **register**.
    """
    assert name in globals(), u'Object {} is not registered'.format(name)
    assert globals()[name] is not _empty, (u'"{}" not registered'.format(name))

    return globals()[name]
